︠9814e215-8001-444f-a279-6887bd5178e6i︠
%html
<h2>Brief study related to the Weiss-Weinstein bound, by Ahmed Fasih<br /></h2>
<p>This brief demo studies the $\eta(s, h)$ function described by Weiss and Weinstein in their 1985 IEEE Trans. Info. Theory paper (vol. IT-31, no. 5, p. 680-682).</p>
<p>We want to estimate the frequency of an exponentially-damped sine-wave (the attenuation factor is proportional to the frequency) in noise from deliberately undersampled data. We use aliasing to give us large peaks in the likelihood far away from the true frequency.</p>
<p>Assume uniform priors on $\theta$. Then, give its bounds and a vector containing it. This vector should be fairly fine because we'll do trapezoidal integration over it.</p>

︡98604dd8-39d8-44c1-91a1-c8b77a0abe40︡{"html": "<h2>Brief study related to the Weiss-Weinstein bound, by Ahmed Fasih<br /></h2>\n<p>This brief demo studies the $\\eta(s, h)$ function described by Weiss and Weinstein in their 1985 IEEE Trans. Info. Theory paper (vol. IT-31, no. 5, p. 680-682).</p>\n<p>We want to estimate the frequency of an exponentially-damped sine-wave (the attenuation factor is proportional to the frequency) in noise from deliberately undersampled data. We use aliasing to give us large peaks in the likelihood far away from the true frequency.</p>\n<p>Assume uniform priors on $\\theta$. Then, give its bounds and a vector containing it. This vector should be fairly fine because we'll do trapezoidal integration over it.</p>"}︡
︠5cedeeed-7407-41bc-b823-a7c28b12c632︠
import numpy

theta_min = 0.
theta_max =  40.
theta = numpy.linspace(theta_min,theta_max, 250)
︡781804ef-c1c3-4f29-ab6e-90b40cc41e5c︡︡
︠23e50600-f1d7-4212-8fc9-e68975f7606bi︠
%html
<p>Next, set up the signal's sample rates and times. We deliberately undersample.</p>

︡3c87fd90-9d88-43b1-b9eb-865796322667︡{"html": "<p>Next, set up the signal's sample rates and times. We deliberately undersample.</p>"}︡
︠6bc6cde9-a35a-49e4-8dcb-d7c346b7e7e6︠
fs=10.
Tmax = 1.
t=numpy.arange(0, Tmax, 1/fs)
︡5c0c14eb-ef9d-44ca-9663-b0eb67097480︡︡
︠ad358688-436d-4e7f-8a9f-5d186da0a43bi︠
%html
<p>These are our test points $\lbrace h \rbrace$. We let them be a grid for plotting purposes, and let their extents be different from those of $\theta$ because we can.</p>

︡0e9ccc9f-54ec-42d6-8c03-5fe2fa1d9ebf︡{"html": "<p>These are our test points $\\lbrace h \\rbrace$. We let them be a grid for plotting purposes, and let their extents be different from those of $\\theta$ because we can.</p>"}︡
︠f65ed5d3-7cef-426e-bc30-6995d67d197c︠
hs = numpy.arange(0, 25, 0.1)
︡a6fc7219-1dc7-4f7a-86c2-ec99dce6e3c4︡︡
︠2455f2f6-f95c-421b-8a05-650a3f0d0fd7i︠
%html
<p>This is our true signal, $s(t)$, an exponentially-damped sine wave. The full data model would thus be</p>
<p>$$x(t)=\sin(2\pi\theta t) \exp(-0.02 \theta t) + \text{AWGN}$$</p>
<p><span style="font-size: small;">[Nota bene: Numpy's sin() will return a matrix or an array depending on what type the time vector "t" is. The distinction between array and matrix types is more important here than in Matlab (where it is non-existant) because * does element-wise multiply for arrays, and does matrix product for matrixes.]</span></p>

︡77dfa066-4a67-4015-a7c5-76e2348a0797︡{"html": "<p>This is our true signal, $s(t)$, an exponentially-damped sine wave. The full data model would thus be</p>\n<p>$$x(t)=\\sin(2\\pi\\theta t) \\exp(-0.02 \\theta t) + \\text{AWGN}$$</p>\n<p><span style=\"font-size: small;\">[Nota bene: Numpy's sin() will return a matrix or an array depending on what type the time vector \"t\" is. The distinction between array and matrix types is more important here than in Matlab (where it is non-existant) because * does element-wise multiply for arrays, and does matrix product for matrixes.]</span></p>"}︡
︠fe9a04f2-ccdf-4079-9357-163adde8a0a6︠
print "Make sure 't' has type array, so * does element-wise multiply:", type(t)
s = lambda x: numpy.sin(t*x*2.*numpy.pi) * numpy.exp(-t*x*.02)
︡dbd85319-3526-4c7c-86f3-38aa33345d97︡{"stdout": "Make sure 't' has type array, so * does element-wise multiply: <type 'numpy.ndarray'>"}︡
︠9327dc19-7a85-46f7-a23d-6984b5d06a20i︠
%html
<p>Here, we assume $s=\frac{1}{2}$ and unit noise variance (that's why there's no "sigma_n" variable), and  evaluate $\eta(h)$ for the range of $h$ specified previously. Each evaluation thereof involves an integral:</p>
<p>$$\begin{align}\eta(h) &amp;= \int_\Theta \sqrt{p(\theta+h) p(\theta)} \int_\Omega \sqrt{p(x|\theta) p(x|\theta+h)} dx d\theta \\</p>
<p>&amp;\propto \int_\Theta \exp \left( -\frac{1}{4 \sigma_n^2} || s(\theta+h) - s(\theta) ||^2 \right) d\theta</p>
<p>\end{align}$$</p>

︡47fd11b6-eb77-483d-8f84-b3ab86f445ac︡{"html": "<p>Here, we assume $s=\\frac{1}{2}$ and unit noise variance (that's why there's no \"sigma_n\" variable), and  evaluate $\\eta(h)$ for the range of $h$ specified previously. Each evaluation thereof involves an integral:</p>\n<p>$$\\begin{align}\\eta(h) &amp;= \\int_\\Theta \\sqrt{p(\\theta+h) p(\\theta)} \\int_\\Omega \\sqrt{p(x|\\theta) p(x|\\theta+h)} dx d\\theta \\\\</p>\n<p>&amp;\\propto \\int_\\Theta \\exp \\left( -\\frac{1}{4 \\sigma_n^2} || s(\\theta+h) - s(\\theta) ||^2 \\right) d\\theta</p>\n<p>\\end{align}$$</p>"}︡
︠d593ecda-07d8-4bd6-8d98-9802eb75b97b︠
from numpy.linalg import norm

def eta(h):
    return numpy.trapz( 
    [numpy.exp(-1/4.*norm(s(th+h)-s(th))**2.) for th in theta] , # NB: this is a list!
    theta)

all_eta = [eta(h) for h in hs] # NB: this is a list!
︡2880bc7b-7e78-4638-9bfc-7fca7261b41d︡︡
︠63c8f0b9-958a-4bc8-9d6d-10382b8e3181i︠
%html
<p>Having stored $\eta(h, s=\frac{1}{2})$ in the above list, we plot below. The reason we point out the lists is that they are not arrays or matrixes, and arithmetic operators on them will fail. <span style="font-size: small;">[We could convert them all to arrays with "numpy.array(...)" but both numpy.trapz() above and pylab.plot() below work fine with lists. As a sidenote, these functions accept a variety of inputs like lists, matrixes, arrays, etc., because they are all members of the same class family and are iteratable.]</span></p>

︡a11bb992-9f6c-49d7-8ae9-a11ecea2937d︡{"html": "<p>Having stored $\\eta(h, s=\\frac{1}{2})$ in the above list, we plot below. The reason we point out the lists is that they are not arrays or matrixes, and arithmetic operators on them will fail. <span style=\"font-size: small;\">[We could convert them all to arrays with \"numpy.array(...)\" but both numpy.trapz() above and pylab.plot() below work fine with lists. As a sidenote, these functions accept a variety of inputs like lists, matrixes, arrays, etc., because they are all members of the same class family and are iteratable.]</span></p>"}︡
︠cc4d9be3-6f3d-47e6-83e1-bae443c84863︠
from matplotlib import rc
rc('text', usetex=True)

import pylab
pylab.figure()
pylab.plot(hs, all_eta, '.-')

pylab.xlabel('$h$', fontsize=18)
pylab.ylabel('$\propto \eta(h, s=0.5)$', fontsize=18)
pylab.title('$\propto \eta(h, s=0.5)$, with uninformative priors', fontsize=18)
pylab.savefig('etas', dpi=72)
︡93cf0e90-2615-476e-9263-f0249a72e290︡︡
